from __future__ import absolute_import
from __future__ import print_function
try:
    import numpy
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import matplotlib.gridspec as gridspec
    import_success = True

except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

def paragraph(text, width=None):
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)

class DataPlotterManager:

    recipe_name = "espda_compu_rhk"
    
    def setInteractiveParameters(self):
        paramList = list()
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "bv", group="all",
            description=" B-V color of the star. [0.82]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
        displayName = "rv",  group="all",
        description="Star's RV. [-9999.0]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
        displayName = "ca_alpha",  group="all",
        description="Calcium index alpha. [1.0]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
        displayName = "ca_beta",  group="all",
        description="Calcium index beta. [0.0]"))


        return paramList

    def readFitsData(self, fitsFiles):
        self.wavel_range = list()
        self.frames = dict()
        for frame in fitsFiles:

            if frame == '':
                continue
            category = frame.category
            self.frames[category] = frame
        self.fits = PipelineProduct(self.frames["RHK_FLUX"])             
        
        self.rhk_flux = PlotableData(self.frames["RHK_FLUX"], x="wavelength", y="flux", mode="plot")
 

    def addSubplots(self, figure):
        gs_rhk_flux1 = gridspec.GridSpec(1, 1, bottom = 0.55, top = 0.95)
        gs_rhk_flux2 = gridspec.GridSpec(1, 1, bottom = 0.05, top = 0.45)
        self.subplot_rhk_flux1 = figure.add_subplot(gs_rhk_flux1[0,0])
        self.subplot_rhk_flux2 = figure.add_subplot(gs_rhk_flux2[0,0])

    def plotProductsGraphics(self):
        self.plotSpec()
        self.firstClick = False

    def plotSpec(self):
        rhk_flux_title1   = "Spectrum (Ca II H)"
        rhk_flux_title2   = "Spectrum (Ca II K)"
        rhk_flux_tooltip = "Green: rebinned data; blue: equalized data.\n" \
                       "Click twice to change the wavelength range.\n" \
                       "Double-click to use the full wavelength range."
        rhk_flux_y_label = r"Flux density [erg $\mathdefault{cm^2}$ " \
                       r"$\mathdefault{s^{-1}}$ $\mathdefault{nm^{-1}}$]"

        all_x_label  = "Wavelength [nm]"
        y = self.fits.readTableColumn(1, "flux")
        x = self.fits.readTableColumn(1, "wavelength")
        #range_tmp = [x <= 3938 ]
        #range = [x[range_tmp] >= 3928 ]
        #y_max = 1.1 * numpy.nanmax(y[range])
        where = numpy.logical_and(x >= 3928, x <= 3938)
        y_max = 1.1 * numpy.nanmax(y[where])
        
        self.rhk_flux.plot(self.subplot_rhk_flux1, title = rhk_flux_title1,
            tooltip = rhk_flux_tooltip, x_label = all_x_label, 
            y_label = rhk_flux_y_label, x_min=3928, x_max=3938, y_max=y_max,
            bars = self.wavel_range)
        
        #range_tmp = [x <= 3973]
        #range = [x[range_tmp] >= 3963 ]
        #y_max = 1.1 * numpy.nanmax(y[range])
        where = numpy.logical_and(x >= 3963, x <= 3973)
        y_max = 1.1 * numpy.nanmax(y[where])
        self.rhk_flux.plot(self.subplot_rhk_flux2, title = rhk_flux_title2,
            tooltip = rhk_flux_tooltip, x_label = all_x_label, 
            y_label = rhk_flux_y_label, x_min=3963, x_max=3973,y_max=y_max,
            bars = self.wavel_range)
        

    def plotWidgets(self):
        widgets = list()
        self.clicks = 0
        self.clickable_rhk_flux = InteractiveClickableSubplot(self.subplot_rhk_flux1,self.wavelRange)
        widgets.append(self.clickable_rhk_flux)
        return widgets

    def wavelRange(self, point):
        self.clicks = self.clicks + 1
        range_limit = self.nord_spec.getXPoint(point.xdata)        
        if self.clicks < 3:
            if range_limit in self.wavel_range:
                self.wavel_range.remove(range_limit)
            else:
                self.wavel_range.append(range_limit)
        self.plotSpec()
        new_params = list()

        return new_params




    def setWindowTitle(self):
      title = 'ESPRESSO Data Analysis Software: Stellar Activity Determination' 
      return title


if __name__ == '__main__':
    import reflex_interactive_app
    import sys
    import pkg_resources
    from espda_plot_common import *
    interactive_app = reflex_interactive_app.PipelineInteractiveApp(
        enable_init_sop = True)
    interactive_app.parse_args()
    if not import_success:
        interactive_app.setEnableGUI(False)
    if interactive_app.isGUIEnabled():
        dataPlotManager = DataPlotterManager()
        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()
    interactive_app.print_outputs()
    sys.exit()
