from __future__ import absolute_import
from __future__ import print_function
try:
    import numpy
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import matplotlib.gridspec as gridspec
    import_success = True

except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

def paragraph(text, width=None):
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)

class DataPlotterManager:

    recipe_name = "espda_compu_starpar"
    
    def setInteractiveParameters(self):
        paramList = list()
        return paramList

    def readFitsData(self, fitsFiles):
        self.wavel_range = list()
        self.frames = dict()
        for frame in fitsFiles:

            if frame == '':
                continue
            category = frame.category
            self.frames[category] = frame
 
        self.teff = PlotableData(self.frames["TEFF"], mode="scatter", x="TEFF", y="STD_TEFF")
        self.feh  = PlotableData(self.frames["FEH"], mode="scatter", x="FEH", y = "STD_FEH")
 

    def addSubplots(self, figure):
        gs_teff = gridspec.GridSpec(1, 1, bottom = 0.45, top = 0.95)
        gs_feh  = gridspec.GridSpec(1, 1, bottom = 0.05, top = 0.35)
        self.subplot_teff = figure.add_subplot(gs_teff[0,0])
        self.subplot_feh  = figure.add_subplot(gs_feh [0,0])

    def plotProductsGraphics(self):
        self.plotSpec()
        self.firstClick = False

    def plotSpec(self):
        teff_title   = "Spectrum"
        teff_tooltip = "Green: rebinned data; blue: equalized data.\n" \
                       "Click twice to change the wavelength range.\n" \
                       "Double-click to use the full wavelength range."
        teff_y_label = r"Flux density [erg $\mathdefault{cm^2}$ " \
                       r"$\mathdefault{s^{-1}}$ $\mathdefault{nm^{-1}}$]"

        feh_title    = "SNR per pixel"
        feh_tooltip  = "Grey: non-rebinned data; green: rebinned data; blue: "\
                       "equalized data." 
        feh_y_label  = r"SNR / pixel size [$\mathdefault{nm^{-1}}$]"
        all_x_label  = "Wavelength [nm]"
                      
 
        self.teff.plot(self.subplot_teff, title = teff_title,
            tooltip = teff_tooltip, x_label = all_x_label, 
            y_label = teff_y_label, bars = self.wavel_range)
        

        self.feh.plot(self.subplot_feh, title = feh_title, 
            tooltip = feh_tooltip, x_label = all_x_label, y_label = feh_y_label)

    def plotWidgets(self):
        widgets = list()
        self.clicks = 0
        self.clickable_teff = InteractiveClickableSubplot(self.subplot_teff, 
            self.wavelRange)
        widgets.append(self.clickable_teff)
        return widgets

    def wavelRange(self, point):
        self.clicks = self.clicks + 1
        range_limit = self.feh.getXPoint(point.xdata)        
        if self.clicks < 3:
            if range_limit in self.wavel_range:
                self.wavel_range.remove(range_limit)
            else:
                self.wavel_range.append(range_limit)
        self.plotSpec()
        new_params = list()
        if self.clicks == 2:
            if len(self.wavel_range) == 2:
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-min", 
                    value = str(self.wavel_range[0])))
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-max", 
                    value = str(self.wavel_range[1])))
            else:        
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-min", 
                    value = str(380.0)))
                new_params.append(reflex.RecipeParameter(
                    recipe = self.recipe_name, displayName = "wavel-max",
                    value = str(780.0)))
        return new_params

if __name__ == '__main__':
    import reflex_interactive_app
    import sys
    import pkg_resources
    from espda_plot_common import *
    interactive_app = reflex_interactive_app.PipelineInteractiveApp(
        enable_init_sop = True)
    interactive_app.parse_args()
    if not import_success:
        interactive_app.setEnableGUI(False)
    if interactive_app.isGUIEnabled():
        dataPlotManager = DataPlotterManager()
        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()
    interactive_app.print_outputs()
    sys.exit()
