from __future__ import absolute_import
from __future__ import print_function
try:
    import numpy as np
    import reflex
    from pipeline_product import PipelineProduct
    import pipeline_display
    import matplotlib.gridspec as gridspec
    import_success = True

except ImportError:
    import_success = False
    print("Error importing modules pyfits, wx, matplotlib, numpy")

def paragraph(text, width=None):
    import textwrap
    if width is None:
        return textwrap.dedent(text).replace('\n', ' ').strip()
    else:
        return textwrap.fill(textwrap.dedent(text), width=width)

class DataPlotterManager:

    recipe_name = "espda_rv_synth"
    
    def setInteractiveParameters(self):
        paramList = list()
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "ccf-cen", group = "ccf", description =
            "Guess radial velocity [km/s]: [-850., +850.]. [0.0]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name, 
            displayName = "ccf-win",  group = "ccf", description =
            "Radial velocity window [km/s]: [10., +1000.]. [350.0]"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "ccf-fitg", group = "ccf", description =
            "Flag to use a Gaussian to fit the CCF (TRUE), or use a "
            "Lorentzian (FALSE: default)"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "ccf-frv", group = "ccf", description =
            "Flag to use the RV derived from CCF fit (TRUE), or from "
            "CCF core fit (FALSE: default)"))
        paramList.append(reflex.RecipeParameter(recipe = self.recipe_name,
            displayName = "syn-fct",  group = "syn", description =
            "Synthesis multiplication factor: [0.01, +1000.]. [1.0]"))
        return paramList

    def readFitsData(self, fitsFiles):
        self.wavel_range = list()
        self.frames = dict()
        #print fitsFiles
        for frame in fitsFiles:
            if frame == '':
                continue
            category = frame.category
            self.frames[category] = frame
        #print self.frames
        #Reading the RV from Keyword:
        self.resf = PipelineProduct(self.frames["RMERG_SPEC"]).\
            readKeyword("ESO DAS QC RVSYNTH STDEV RESIC W680")

        self.rvccf = PipelineProduct(self.frames["RMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC RVSYNTH RVCCF")
        self.rveccf = PipelineProduct(self.frames["RMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC RVSYNTH RVECCF")
        #Reading the RV from Keyword:
        self.rvcore = PipelineProduct(self.frames["RMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC RVSYNTH RVCORE")
        self.rvecore = PipelineProduct(self.frames["RMERG_SPEC"]).\
            readKeyword("HIERARCH ESO DAS QC RVSYNTH RVECORE")
        
        self.spec = PlotableData(self.frames["RMERG_SPEC"], x = "WAVEC", y = "FLUX")

        self.spec_synth = PlotableData(self.frames["RMERG_SPEC"], y = "SYNTC",
                                       ref_fits = self.frames["RMERG_SPEC"],
                                       color = "red", mode = "plot")

        self.spec_res = PlotableData(self.frames["RMERG_SPEC"], x = "WAVEC",
                                     ref_fits = self.frames["RMERG_SPEC"],
                                     x_ref = "WAVEL" , y_ref = "FLUX",
                                     y = "RESIC", color = "green",
                                     mode = "plot")
    
        self.ccf = PlotableData(self.frames["CCF_SPEC"],
                                     x = "XCCF", y = "YCCF",
                                     x_ref = "XCCF" , y_ref = "YCCF",
                                     mode = "scatter", marker=".",
                                     width = 0.5, size = 40)
                                     #mode = "plot")

        self.ccf_fit = PlotableData(self.frames["CCF_SPEC"], x = "XCCF",
                                         x_ref = "XCCF" , y_ref = "YCCF",
                                         y = "YFIT", color = "red",
                                         mode = "plot")

        self.core = PlotableData(self.frames["CCF_SPEC"],
                        x = "XCCF", y = "YCCF",
                        x_ref = "XCCF" , y_ref = "YCCF",
                        mode = "scatter", marker=".",
                        width = 0.5, size = 40)

        self.core_fcore = PlotableData(self.frames["CCF_SPEC"], x = "XCCF",
                                 x_ref = "XCCF" , y_ref = "YCCF",
                                 y = "YCORE", color = "green",
                                 mode = "plot")

    def addSubplots(self, figure):
        gs_spec = gridspec.GridSpec(1, 1, bottom = 0.60, top = 0.95)
        gs_res  = gridspec.GridSpec(1, 1, bottom = 0.35, top = 0.50)
        gs_ccf  = gridspec.GridSpec(1, 1, left=0.10, right= 0.55,
                                    bottom = 0.10, top = 0.25)
        gs_core  = gridspec.GridSpec(1, 1, left=0.70, right= 0.95,
                                     bottom = 0.10, top = 0.25)
        self.subplot_spec = figure.add_subplot(gs_spec[0,0])
        self.subplot_res  = figure.add_subplot(gs_res[0,0])
        self.subplot_ccf  = figure.add_subplot(gs_ccf[0,0])
        self.subplot_core = figure.add_subplot(gs_core[0,0])
        
    def plotProductsGraphics(self):
        self.plotSpec()
        self.firstClick = False

    def plotSpec(self):
        spec_title    = "Observed and Synthetic spectra"
        spec_tooltip  = "Blue: rebinned normalized spectrum; red: synthetic spectrum"
        spec_y_label  = r"Normalized flux"
        all_x_label  = "Wavelength [nm]"

        res_title    = "Residuals RV-corrected Observed - Synthetic spectra"
        res_tooltip  = "Green: residuals: observed-synthetic fluxes"
        res_y_label  = r"Flux residuals"
        
        ccf_title    = "Cross correlation function (CCF)"
        ccf_tooltip  = "Blue: normalized cross-correlation function (CCF); red: CCF fit"

        core_title    = "CCF core"
        core_tooltip  = "Blue: normalized cross-correlation function (CCF); green: CCF fit"

        ccf_y_label  = r"CCF"
        ccf_x_label  = "Radial velocity [km/s]"

        self.spec.plot(self.subplot_spec, title = spec_title,
                             tooltip = spec_tooltip, x_label = all_x_label,
                             y_min=-0.1,y_max=1.2,
                             y_label = spec_y_label, bars = self.wavel_range)
                             
        self.spec_synth.plot(self.subplot_spec, title = spec_title,
                             tooltip = spec_tooltip, y_min=-0.1,y_max=1.2,
                             x_label = all_x_label, y_label = spec_y_label)
 
        ref_fits = self.frames["RMERG_SPEC"]
        self.ref_fits = PipelineProduct(ref_fits)
        f = self.ref_fits.readTableColumn(1, "FLUX")
        s = self.ref_fits.readTableColumn(1, "SYNTH")
        res = f - s
        #      res = self.ref_fits.readTableColumn(1, "FLUX")
        ymaxr = np.nanmax(res)
        yminr = np.nanmin(res)
        dyr = (ymaxr - yminr) / 50.

        self.spec_res.plot(self.subplot_res, title = res_title,
                           tooltip = res_tooltip,
                           #y_min = yminr-dyr, y_max = ymaxr+dyr,
                           y_min=-1.-dyr,y_max=1+dyr,
                           x_label = all_x_label, y_label = res_y_label)

        #CCF plot
        if "CCF_SPEC"  in self.frames:
            self.fits = PipelineProduct(self.frames["CCF_SPEC"])
        y = self.fits.readTableColumn(1, "YCCF")
        ymax = np.nanmax(y)
        ymin = np.nanmin(y)
        dy = (ymax - ymin)/10.

        self.ccf.plot(self.subplot_ccf, title= ccf_title,
                           tooltip=ccf_tooltip,
                           y_min=ymin-dy,y_max=(ymax+dy)*1.02,
                           x_label=ccf_x_label, y_label=ccf_y_label)

        self.ccf_fit.plot(self.subplot_ccf, title = ccf_title,
                               tooltip = ccf_tooltip, x_label = ccf_x_label,
                               y_min=ymin-dy,y_max=(ymax+dy)*1.02,
                               y_label = ccf_y_label)

        #Core plot
        yc = self.fits.readTableColumn(1, "YCORE")
        xc = self.fits.readTableColumn(1, "XCCF")
            #ic = find(yc > 0. )
            #ycf = yc[ic]
            #xcf = xcf[ic]
            #xcf = xc[yc > 0.]
            #ycf = yc > 0.
        xcf = xc[np.where(yc > 0.)]
        ycf = yc[np.where(yc > 0.)]
        ycmax = np.nanmax(ycf)
        ycmin = np.nanmin(ycf)
        xcmax = np.nanmax(xcf)
        xcmin = np.nanmin(xcf)
        dyc = (ymax - ymin)/50.
        dxc = xcf[1] - xcf[0]

        self.core.plot(self.subplot_core, title= core_title,
              tooltip=core_tooltip,
              y_min=ycmin+dyc*2.5,y_max=(ycmax+dyc)*1.02,
              x_min=xcmin+dxc/2.,x_max=xcmax-dxc/2.,
              x_label=ccf_x_label, y_label=ccf_y_label)
    
        self.core_fcore.plot(self.subplot_core, title = core_title,
                  tooltip = core_tooltip,
                  y_min=ycmin+dyc*2.5,y_max=(ycmax+dyc)*1.02,
                  x_min=xcmin+dxc/2.,x_max=xcmax-dxc/2.,
                  x_label=ccf_x_label, y_label=ccf_y_label)
    
        #Text plot
        self.subplot_res.text(0.95, 0.75, r"RMS($\lambda$<680nm) = " +
                       str(np.around(self.resf, decimals=3)),
                       horizontalalignment="right",
                       verticalalignment="bottom",
                       fontsize=8, transform=self.subplot_res.transAxes)

        self.subplot_ccf.text(0.95, 0.8, r"RV: = " +
                       str(np.around(self.rvccf, decimals=3)),
                       horizontalalignment="right",
                       verticalalignment="bottom",
                       fontsize=8, transform=self.subplot_ccf.transAxes)

        self.subplot_ccf.text(0.95, 0.7, r"eRV: = " +
                      str(np.around(self.rveccf, decimals=3)),
                      horizontalalignment="right",
                      verticalalignment="bottom",
                      fontsize=8, transform=self.subplot_ccf.transAxes)

        self.subplot_core.text(0.75, 0.15, r"RV: = " +
                               str(np.around(self.rvcore, decimals=3)),
                               horizontalalignment="right",
                               verticalalignment="bottom",
                               fontsize=8, transform=self.subplot_core.transAxes)

        self.subplot_core.text(0.75, 0.05, r"eRV: = " +
                       str(np.around(self.rvecore, decimals=3)),
                       horizontalalignment="right",
                       verticalalignment="bottom",
                       fontsize=8, transform=self.subplot_core.transAxes)
    
    def plotWidgets(self):
        widgets = list()
        self.clicks = 0
        self.clickable_spec = InteractiveClickableSubplot(self.subplot_spec, 
            self.wavelRange)
        widgets.append(self.clickable_spec)
        return widgets

    def wavelRange(self, point):
        self.clicks = self.clicks + 1
        range_limit = self.spec.getXPoint(point.xdata)        
        if self.clicks < 3:
            if range_limit in self.wavel_range:
                self.wavel_range.remove(range_limit)
            else:
                self.wavel_range.append(range_limit)
        self.plotSpec()
        return new_params

    def setWindowTitle(self):
      title = 'ESPRESSO Data Analysis Software: Radial Velocity Computation'
      return title

if __name__ == '__main__':
    import reflex_interactive_app
    import sys
    import pkg_resources
    from espda_plot_common import *
    interactive_app = reflex_interactive_app.PipelineInteractiveApp(
        enable_init_sop = True)
    interactive_app.parse_args()
    if not import_success:
        interactive_app.setEnableGUI(False)
    if interactive_app.isGUIEnabled():
        dataPlotManager = DataPlotterManager()
        interactive_app.setPlotManager(dataPlotManager)
        interactive_app.showGUI()
    else:
        interactive_app.set_continue_mode()
    interactive_app.print_outputs()
    sys.exit()
